@@ -1,13 +1,14 @@
 # Human friendly input/output in Python.
 #
 # Author: Peter Odding <peter@peterodding.com>
-# Last Change: March 31, 2018
+# Last Change: July 14, 2018
 # URL: https://humanfriendly.readthedocs.io
 
 """The main module of the `humanfriendly` package."""
 
 # Standard library modules.
 import collections
+import datetime
 import decimal
 import multiprocessing
 import numbers
@@ -39,7 +40,7 @@ from humanfriendly.prompts import prompt_for_choice  # NOQA
 from humanfriendly.compat import is_string, monotonic
 
 # Semi-standard module versioning.
-__version__ = '4.10'
+__version__ = '4.15'
 
 # Spinners are redrawn at most this many seconds.
 minimum_spinner_interval = 0.2
@@ -135,6 +136,29 @@ def coerce_pattern(value, flags=0):
     return value
 
 
+def coerce_seconds(value):
+    """
+    Coerce a value to the number of seconds.
+
+    :param value: An :class:`int`, :class:`float` or
+                  :class:`datetime.timedelta` object.
+    :returns: An :class:`int` or :class:`float` value.
+
+    When `value` is a :class:`datetime.timedelta` object the
+    :func:`~datetime.timedelta.total_seconds()` method is called.
+    On Python 2.6 this method is not available so it is emulated.
+    """
+    if isinstance(value, datetime.timedelta):
+        if hasattr(value, 'total_seconds'):
+            return value.total_seconds()
+        else:
+            return (value.microseconds + (value.seconds + value.days * 24 * 3600) * 10**6) / 10**6
+    if not isinstance(value, numbers.Number):
+        msg = "Failed to coerce value to number of seconds! (%r)"
+        raise ValueError(format(msg, value))
+    return value
+
+
 def format_size(num_bytes, keep_width=False, binary=False):
     """
     Format a byte count as a human readable file size.
@@ -375,13 +399,14 @@ def format_timespan(num_seconds, detailed=False, max_units=3):
     """
     Format a timespan in seconds as a human readable string.
 
-    :param num_seconds: Number of seconds (integer or float).
+    :param num_seconds: Any value accepted by :func:`coerce_seconds()`.
     :param detailed: If :data:`True` milliseconds are represented separately
                      instead of being represented as fractional seconds
                      (defaults to :data:`False`).
     :param max_units: The maximum number of units to show in the formatted time
                       span (an integer, defaults to three).
     :returns: The formatted timespan as a string.
+    :raise: See :func:`coerce_seconds()`.
 
     Some examples:
 
@@ -399,6 +424,7 @@ def format_timespan(num_seconds, detailed=False, max_units=3):
     >>> format_timespan(week * 52 + day * 2 + hour * 3)
     '1 year, 2 days and 3 hours'
     """
+    num_seconds = coerce_seconds(num_seconds)
     if num_seconds < 60 and not detailed:
         # Fast path.
         return pluralize(round_number(num_seconds), 'second')
