@@ -4,20 +4,20 @@
 # Tests for the `humanfriendly' package.
 #
 # Author: Peter Odding <peter.odding@paylogic.eu>
-# Last Change: March 28, 2018
+# Last Change: July 14, 2018
 # URL: https://humanfriendly.readthedocs.io
 
 """Test suite for the `humanfriendly` package."""
 
 # Standard library modules.
+import datetime
 import math
 import os
-import re
 import random
+import re
 import subprocess
 import sys
 import time
-import unittest
 
 # Modules included in our package.
 import humanfriendly
@@ -54,6 +54,7 @@ from humanfriendly.terminal import (
     connected_to_terminal,
     find_terminal_size,
     get_pager_command,
+    html_to_ansi,
     message,
     output,
     show_pager,
@@ -291,6 +292,11 @@ class HumanFriendlyTestCase(TestCase):
         assert '5 minutes and 0.3 seconds' == humanfriendly.format_timespan(300.300)
         assert '1 second and 15 milliseconds' == humanfriendly.format_timespan(1.015, detailed=True)
         assert '10 seconds and 15 milliseconds' == humanfriendly.format_timespan(10.015, detailed=True)
+        # Test the datetime.timedelta support:
+        # https://github.com/xolox/python-humanfriendly/issues/27
+        now = datetime.datetime.now()
+        then = now - datetime.timedelta(hours=23)
+        assert '23 hours' == humanfriendly.format_timespan(now - then)
 
     def test_parse_timespan(self):
         """Test :func:`humanfriendly.parse_timespan()`."""
@@ -766,11 +772,28 @@ class HumanFriendlyTestCase(TestCase):
         # Test `humanfriendly --run-command'.
         returncode, output = run_cli(main, '--run-command', 'bash', '-c', 'sleep 2 && exit 42')
         assert returncode == 42
+        # Test `humanfriendly --demo'. The purpose of this test is
+        # to ensure that the demo runs successfully on all versions
+        # of Python and outputs the expected sections (recognized by
+        # their headings) without triggering exceptions. This was
+        # written as a regression test after issue #28 was reported:
+        # https://github.com/xolox/python-humanfriendly/issues/28
+        returncode, output = run_cli(main, '--demo')
+        assert returncode == 0
+        lines = [ansi_strip(l) for l in output.splitlines()]
+        assert "Text styles:" in lines
+        assert "Foreground colors:" in lines
+        assert "Background colors:" in lines
+        assert "256 color mode (standard colors):" in lines
+        assert "256 color mode (high-intensity colors):" in lines
+        assert "256 color mode (216 colors):" in lines
+        assert "256 color mode (gray scale colors):" in lines
 
     def test_ansi_style(self):
         """Test :func:`humanfriendly.terminal.ansi_style()`."""
         assert ansi_style(bold=True) == '%s1%s' % (ANSI_CSI, ANSI_SGR)
         assert ansi_style(faint=True) == '%s2%s' % (ANSI_CSI, ANSI_SGR)
+        assert ansi_style(italic=True) == '%s3%s' % (ANSI_CSI, ANSI_SGR)
         assert ansi_style(underline=True) == '%s4%s' % (ANSI_CSI, ANSI_SGR)
         assert ansi_style(inverse=True) == '%s7%s' % (ANSI_CSI, ANSI_SGR)
         assert ansi_style(strike_through=True) == '%s9%s' % (ANSI_CSI, ANSI_SGR)
@@ -779,6 +802,9 @@ class HumanFriendlyTestCase(TestCase):
         assert ansi_style(color='blue', bright=True) == '%s94%s' % (ANSI_CSI, ANSI_SGR)
         assert ansi_style(color=214) == '%s38;5;214%s' % (ANSI_CSI, ANSI_SGR)
         assert ansi_style(background=214) == '%s39;5;214%s' % (ANSI_CSI, ANSI_SGR)
+        assert ansi_style(color=(0, 0, 0)) == '%s38;2;0;0;0%s' % (ANSI_CSI, ANSI_SGR)
+        assert ansi_style(color=(255, 255, 255)) == '%s38;2;255;255;255%s' % (ANSI_CSI, ANSI_SGR)
+        assert ansi_style(background=(50, 100, 150)) == '%s48;2;50;100;150%s' % (ANSI_CSI, ANSI_SGR)
         self.assertRaises(ValueError, ansi_style, color='unknown')
 
     def test_ansi_width(self):
@@ -805,6 +831,48 @@ class HumanFriendlyTestCase(TestCase):
         # Make sure ansi_wrap() ends the text by resetting the ANSI styles.
         assert ansi_wrap(text, bold=True).endswith(ANSI_RESET)
 
+    def test_html_to_ansi(self):
+        """Test the :func:`humanfriendly.terminal.html_to_ansi()` function."""
+        assert html_to_ansi("Just some plain text") == "Just some plain text"
+        # Hyperlinks.
+        assert html_to_ansi('<a href="https://python.org">python.org</a>') == \
+            '\x1b[4;94mpython.org\x1b[0m (\x1b[4;94mhttps://python.org\x1b[0m)'
+        # Make sure `mailto:' prefixes are stripped (they're not at all useful in a terminal).
+        assert html_to_ansi('<a href="mailto:peter@peterodding.com">peter@peterodding.com</a>') == \
+            '\x1b[4;94mpeter@peterodding.com\x1b[0m'
+        # Bold text.
+        assert html_to_ansi("Let's try <b>bold</b>") == \
+            ("Let's try %s" % ansi_wrap("bold", bold=True))
+        assert html_to_ansi("Let's try <span style=\"font-weight: bold\">bold</span>") == \
+            ("Let's try %s" % ansi_wrap("bold", bold=True))
+        # Italic text.
+        assert html_to_ansi("Let's try <i>italic</i>") == \
+            ("Let's try %s" % ansi_wrap("italic", italic=True))
+        assert html_to_ansi("Let's try <span style=\"font-style: italic\">italic</span>") == \
+            ("Let's try %s" % ansi_wrap("italic", italic=True))
+        # Underlined text.
+        assert html_to_ansi("Let's try <ins>underline</ins>") == \
+            ("Let's try %s" % ansi_wrap("underline", underline=True))
+        assert html_to_ansi("Let's try <span style=\"text-decoration: underline\">underline</span>") == \
+            ("Let's try %s" % ansi_wrap("underline", underline=True))
+        # Strike-through text.
+        assert html_to_ansi("Let's try <s>strike-through</s>") == \
+            ("Let's try %s" % ansi_wrap("strike-through", strike_through=True))
+        assert html_to_ansi("Let's try <span style=\"text-decoration: line-through\">strike-through</span>") == \
+            ("Let's try %s" % ansi_wrap("strike-through", strike_through=True))
+        # Pre-formatted text.
+        assert html_to_ansi("Let's try <code>pre-formatted</code>") == \
+            ("Let's try %s" % ansi_wrap("pre-formatted", color='yellow'))
+        # Text colors (with a 6 digit hexadecimal color value).
+        assert html_to_ansi("Let's try <span style=\"color: #AABBCC\">text colors</s>") == \
+            ("Let's try %s" % ansi_wrap("text colors", color=(0xAA, 0xBB, 0xCC)))
+        # Background colors (with an rgb(N, N, N) expression).
+        assert html_to_ansi("Let's try <span style=\"background-color: rgb(50, 50, 50)\">background colors</s>") == \
+            ("Let's try %s" % ansi_wrap("background colors", background=(50, 50, 50)))
+        # Line breaks.
+        assert html_to_ansi("Let's try some<br>line<br>breaks") == \
+            "Let's try some\nline\nbreaks"
+
     def test_generate_output(self):
         """Test the :func:`humanfriendly.terminal.output()` function."""
         text = "Standard output generated by output()"
@@ -1171,7 +1239,3 @@ def normalize_timestamp(value, ndigits=1):
     multitasking, processor scheduling, etc.
     """
     return '%.2f' % round(float(value), ndigits=ndigits)
-
-
-if __name__ == '__main__':
-    unittest.main()
